/*
 Si consideri la Sintassi Astratta sotto, per il Lambda Calcolo:
 Lambda::= Var | Const | [$] Var Lambda | [@] Lambda Lambda
 dove usiamo $ per l'astrazione e @ per l'applicazione.
 ....
 
LastT.c è il modulo delle procedure del modulo header, LastT.h, che fornisce
l'implementazione 
 di 5 nuovi tipi:
 VartT, ConstT, AbsT, AppT, LastT
 I tipi ide e num possono essere direttamente implementati con i tipi string
 e int, in analogia a quanto fatto in laboratorio per i ParseTree in C.
 Il tipo LastT introduce alberi astratti il Lambda Calcolo. Questo ipo deve
 contenere 4 distinti tipi di alberi astratti per i diversi tipi di termini
 del Lambda Calcolo.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "itoa.c"
#include "Last.h"

//Le operazioni su LastT.c:
// mkConst, mkVar, mkAbs, mkApp, isConst, isVar, isAbs, isApp
// printAST, getIde, getTem, getFun, getArg

LastT mkConst(char* c){
	LastT r;
	r = (LastT) malloc(sizeof(struct LambdaTerm));
	r->flag = Const;
	r->term.C.name = c;
	return r;
}
LastT mkVar(char* x){
	LastT r;
	r = (LastT) malloc(sizeof(struct LambdaTerm));
	r->flag = Var;
	r->term.V.name = x;
	return r;
}
LastT mkAbs(char* x,LastT t){
	LastT r;
	r = (LastT) malloc(sizeof(struct LambdaTerm));
	r->flag = Abs;
	r->term.A.name = x;
    r->term.A.term = t;
	return r;
}
LastT mkApp(LastT t1, LastT t2){
	LastT r;
	r = (LastT) malloc(sizeof(struct LambdaTerm));
	r->flag = App;
	r->term.P.fun = t1;
	r->term.P.arg = t2;
	return r;
}
bool isConst(LastT t){
	return(t->flag == Const);
}
bool isVar(LastT t){
	return(t->flag == Var);
}
bool isAbs(LastT t){
	return(t->flag == Abs);
}
bool isApp(LastT t){
	return(t->flag == App);
}
void printAST(LastT t){
	if (isConst(t)){
        printf("[%s]",getIde(t));
        return;
	}
    if (isVar(t)) {
        printf("[%s]",getIde(t));
        return;
    }
    if (isAbs(t)) {
        printf("[$-(%s,",getIde(t));
        printAST(getTerm(t));
        printf(")]");
        return;
    }
    printf("[@ -(");
    printAST(getFun(t));
    printf(",");
    printAST(getArg(t));
    printf(")]");
} 
char* getIde(LastT t){
    if (isConst(t)){
        return t->term.C.name;
    }
    if (isVar(t)) {
        return t->term.V.name;
    }
    return t->term.A.name;
}
LastT getTerm(LastT t){
    return t->term.A.term;
}
LastT getFun(LastT t){
    return t->term.P.fun;
}
LastT getArg(LastT t){
    return t->term.P.arg;
}


